/* ***********************************************************
 * This file was automatically generated on 2019-09-16.      *
 *                                                           *
 * OpenHAB Bindings Version 2.1.23                           *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package com.tinkerforge;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.Arrays;
import java.util.List;
import java.net.URI;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.function.Function;
import java.util.function.BiConsumer;
import org.eclipse.smarthome.config.core.Configuration;
import org.eclipse.smarthome.config.core.ConfigDescription;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameter.Type;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterBuilder;
import org.eclipse.smarthome.config.core.ConfigDescriptionParameterGroup;
import org.eclipse.smarthome.config.core.ParameterOption;
import org.eclipse.smarthome.core.types.State;
import org.eclipse.smarthome.core.types.StateOption;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.core.types.CommandDescriptionBuilder;
import org.eclipse.smarthome.core.types.CommandOption;
import org.eclipse.smarthome.core.thing.ThingTypeUID;
import org.eclipse.smarthome.core.thing.type.ChannelDefinitionBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelType;
import org.eclipse.smarthome.core.thing.type.ChannelTypeBuilder;
import org.eclipse.smarthome.core.thing.type.ChannelTypeUID;
import org.eclipse.smarthome.core.thing.type.ThingType;
import org.eclipse.smarthome.core.thing.type.ThingTypeBuilder;
import org.eclipse.smarthome.core.types.StateDescriptionFragmentBuilder;
import org.eclipse.smarthome.binding.tinkerforge.internal.TinkerforgeBindingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.eclipse.smarthome.core.library.types.QuantityType;
import org.eclipse.smarthome.core.library.unit.MetricPrefix;
import org.eclipse.smarthome.core.library.unit.SIUnits;
import org.eclipse.smarthome.core.library.unit.SmartHomeUnits;
import org.eclipse.smarthome.core.library.types.OnOffType;
import org.eclipse.smarthome.core.library.types.StringType;

/**
 * Capacitive touch sensor for 12 electrodes
 */
public class BrickletMultiTouch extends Device {
	public final static int DEVICE_IDENTIFIER = 234;
	public final static String DEVICE_DISPLAY_NAME = "Multi Touch Bricklet";

    public final static DeviceInfo DEVICE_INFO = new DeviceInfo(DEVICE_DISPLAY_NAME, "multitouch", DEVICE_IDENTIFIER, BrickletMultiTouch.class);

	public final static byte FUNCTION_GET_TOUCH_STATE = (byte)1;
	public final static byte FUNCTION_RECALIBRATE = (byte)2;
	public final static byte FUNCTION_SET_ELECTRODE_CONFIG = (byte)3;
	public final static byte FUNCTION_GET_ELECTRODE_CONFIG = (byte)4;
	public final static byte FUNCTION_SET_ELECTRODE_SENSITIVITY = (byte)6;
	public final static byte FUNCTION_GET_ELECTRODE_SENSITIVITY = (byte)7;
	public final static byte FUNCTION_GET_IDENTITY = (byte)255;
	private final static int CALLBACK_TOUCH_STATE = 5;


	private List<TouchStateListener> listenerTouchState = new CopyOnWriteArrayList<TouchStateListener>();

	/**
	 * Returns the current touch state, see {@link BrickletMultiTouch#getTouchState()} for
	 * information about the state.
	 * 
	 * This listener is triggered every time the touch state changes.
	 */
	public interface TouchStateListener extends DeviceListener {
		public void touchState(int state);
	}

	/**
	 * Creates an object with the unique device ID \c uid. and adds it to
	 * the IP Connection \c ipcon.
	 */
	public BrickletMultiTouch(String uid, IPConnection ipcon) {
		super(uid, ipcon);

		apiVersion[0] = 2;
		apiVersion[1] = 0;
		apiVersion[2] = 0;
		responseExpected[IPConnection.unsignedByte(FUNCTION_GET_TOUCH_STATE)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_RECALIBRATE)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_SET_ELECTRODE_CONFIG)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_GET_ELECTRODE_CONFIG)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_SET_ELECTRODE_SENSITIVITY)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_GET_ELECTRODE_SENSITIVITY)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnection.unsignedByte(FUNCTION_GET_IDENTITY)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;

		callbacks[CALLBACK_TOUCH_STATE] = new IPConnection.DeviceCallbackListener() {
			public void callback(byte[] packet) {
				ByteBuffer bb = ByteBuffer.wrap(packet, 8, packet.length - 8);
				bb.order(ByteOrder.LITTLE_ENDIAN);

				int state = IPConnection.unsignedShort(bb.getShort());

				for (TouchStateListener listener: listenerTouchState) {
					listener.touchState(state);
				}
			}
		};
	}

	/**
	 * Returns the current touch state. The state is given as a bitfield.
	 * 
	 * Bits 0 to 11 represent the 12 electrodes and bit 12 represents
	 * the proximity.
	 * 
	 * If an electrode is touched, the corresponding bit is *true*. If
	 * a hand or similar is in proximity to the electrodes, bit 12 is
	 * *true*.
	 * 
	 * Example: The state 4103 = 0x1007 = 0b1000000000111 means that
	 * electrodes 0, 1 and 2 are touched and that something is in the
	 * proximity of the electrodes.
	 * 
	 * The proximity is activated with a distance of 1-2cm. An electrode
	 * is already counted as touched if a finger is nearly touching the
	 * electrode. This means that you can put a piece of paper or foil
	 * or similar on top of a electrode to build a touch panel with
	 * a professional look.
	 */
	public int getTouchState() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_TOUCH_STATE, this);


		byte[] response = sendRequest(bb.array());

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		int state = IPConnection.unsignedShort(bb.getShort());

		return state;
	}

	/**
	 * Recalibrates the electrodes. Call this function whenever you changed
	 * or moved you electrodes.
	 */
	public void recalibrate() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_RECALIBRATE, this);


		sendRequest(bb.array());
	}

	/**
	 * Enables/disables electrodes with a bitfield (see {@link BrickletMultiTouch#getTouchState()}).
	 * 
	 * *True* enables the electrode, *false* disables the electrode. A
	 * disabled electrode will always return *false* as its state. If you
	 * don't need all electrodes you can disable the electrodes that are
	 * not needed.
	 * 
	 * It is recommended that you disable the proximity bit (bit 12) if
	 * the proximity feature is not needed. This will reduce the amount of
	 * traffic that is produced by the {@link BrickletMultiTouch.TouchStateListener} listener.
	 * 
	 * Disabling electrodes will also reduce power consumption.
	 * 
	 * Default: 8191 = 0x1FFF = 0b1111111111111 (all electrodes enabled)
	 */
	public void setElectrodeConfig(int enabledElectrodes) throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)10, FUNCTION_SET_ELECTRODE_CONFIG, this);

		bb.putShort((short)enabledElectrodes);

		sendRequest(bb.array());
	}

	/**
	 * Returns the electrode configuration, as set by {@link BrickletMultiTouch#setElectrodeConfig(int)}.
	 */
	public int getElectrodeConfig() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_ELECTRODE_CONFIG, this);


		byte[] response = sendRequest(bb.array());

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		int enabledElectrodes = IPConnection.unsignedShort(bb.getShort());

		return enabledElectrodes;
	}

	/**
	 * Sets the sensitivity of the electrodes. An electrode with a high sensitivity
	 * will register a touch earlier then an electrode with a low sensitivity.
	 * 
	 * If you build a big electrode you might need to decrease the sensitivity, since
	 * the area that can be charged will get bigger. If you want to be able to
	 * activate an electrode from further away you need to increase the sensitivity.
	 * 
	 * After a new sensitivity is set, you likely want to call {@link BrickletMultiTouch#recalibrate()}
	 * to calibrate the electrodes with the newly defined sensitivity.
	 * 
	 * The valid sensitivity value range is 5-201.
	 * 
	 * The default sensitivity value is 181.
	 */
	public void setElectrodeSensitivity(short sensitivity) throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)9, FUNCTION_SET_ELECTRODE_SENSITIVITY, this);

		bb.put((byte)sensitivity);

		sendRequest(bb.array());
	}

	/**
	 * Returns the current sensitivity, as set by {@link BrickletMultiTouch#setElectrodeSensitivity(short)}.
	 */
	public short getElectrodeSensitivity() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_ELECTRODE_SENSITIVITY, this);


		byte[] response = sendRequest(bb.array());

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short sensitivity = IPConnection.unsignedByte(bb.get());

		return sensitivity;
	}

	/**
	 * Returns the UID, the UID where the Bricklet is connected to,
	 * the position, the hardware and firmware version as well as the
	 * device identifier.
	 * 
	 * The position can be 'a', 'b', 'c' or 'd'.
	 * 
	 * The device identifier numbers can be found :ref:`here &lt;device_identifier&gt;`.
	 * |device_identifier_constant|
	 */
	public Identity getIdentity() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_IDENTITY, this);


		byte[] response = sendRequest(bb.array());

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		Identity obj = new Identity();
		obj.uid = IPConnection.string(bb, 8);
		obj.connectedUid = IPConnection.string(bb, 8);
		obj.position = (char)(bb.get());
		for (int i = 0; i < 3; i++) {
			obj.hardwareVersion[i] = IPConnection.unsignedByte(bb.get());
		}
		for (int i = 0; i < 3; i++) {
			obj.firmwareVersion[i] = IPConnection.unsignedByte(bb.get());
		}
		obj.deviceIdentifier = IPConnection.unsignedShort(bb.getShort());

		return obj;
	}

	/**
	 * Adds a TouchState listener.
	 */
	public void addTouchStateListener(TouchStateListener listener) {
		listenerTouchState.add(listener);
	}

	/**
	 * Removes a TouchState listener.
	 */
	public void removeTouchStateListener(TouchStateListener listener) {
		listenerTouchState.remove(listener);
	}

    private final Logger logger = LoggerFactory.getLogger(BrickletMultiTouch.class);
    private final static Logger static_logger = LoggerFactory.getLogger(BrickletMultiTouch.class);

    @Override
    public void initialize(org.eclipse.smarthome.config.core.Configuration config, Function<String, org.eclipse.smarthome.config.core.Configuration> getChannelConfigFn, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletMultiTouchConfig cfg = (BrickletMultiTouchConfig) config.as(BrickletMultiTouchConfig.class);
        this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode0", transformMultiTouchElectrode0Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode1", transformMultiTouchElectrode1Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode2", transformMultiTouchElectrode2Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode3", transformMultiTouchElectrode3Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode4", transformMultiTouchElectrode4Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode5", transformMultiTouchElectrode5Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode6", transformMultiTouchElectrode6Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode7", transformMultiTouchElectrode7Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode8", transformMultiTouchElectrode8Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode9", transformMultiTouchElectrode9Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode10", transformMultiTouchElectrode10Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchElectrode11", transformMultiTouchElectrode11Callback0(state, cfg));}});
		this.addTouchStateListener((state) -> {if(true) {updateStateFn.accept("MultiTouchProximity", transformMultiTouchProximityCallback0(state, cfg));}});
        this.setElectrodeSensitivity(cfg.sensitivity.shortValue());this.recalibrate();
		if (cfg.electrode0Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode0").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode1Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode1").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode2Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode2").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode3Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode3").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode4Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode4").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode5Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode5").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode6Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode6").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode7Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode7").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode8Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode8").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode9Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode9").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode10Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode10").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.electrode11Enabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchElectrode11").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		if (cfg.proximityEnabled) {
		BrickletMultiTouchElectrodeConfig channelCfg = getChannelConfigFn.apply("MultiTouchProximity").as(BrickletMultiTouchElectrodeConfig.class);
		
		}
		{
		BrickletMultiTouchRecalibrateConfig channelCfg = getChannelConfigFn.apply("MultiTouchRecalibrate").as(BrickletMultiTouchRecalibrateConfig.class);
		
		}
    }

    @Override
    public void dispose(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException {
        BrickletMultiTouchConfig cfg = (BrickletMultiTouchConfig) config.as(BrickletMultiTouchConfig.class);
        this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
		this.listenerTouchState.clear();
        
		
		
		
		
		
		
		
		
		
		
		
		
		
    }

    @Override
    public List<String> getEnabledChannels(org.eclipse.smarthome.config.core.Configuration config) throws TinkerforgeException{
        BrickletMultiTouchConfig cfg = (BrickletMultiTouchConfig) config.as(BrickletMultiTouchConfig.class);
        List<String> result = new ArrayList<String>();
        if (cfg.electrode0Enabled) {
                result.add("MultiTouchElectrode0");
            }
		if (cfg.electrode1Enabled) {
                result.add("MultiTouchElectrode1");
            }
		if (cfg.electrode2Enabled) {
                result.add("MultiTouchElectrode2");
            }
		if (cfg.electrode3Enabled) {
                result.add("MultiTouchElectrode3");
            }
		if (cfg.electrode4Enabled) {
                result.add("MultiTouchElectrode4");
            }
		if (cfg.electrode5Enabled) {
                result.add("MultiTouchElectrode5");
            }
		if (cfg.electrode6Enabled) {
                result.add("MultiTouchElectrode6");
            }
		if (cfg.electrode7Enabled) {
                result.add("MultiTouchElectrode7");
            }
		if (cfg.electrode8Enabled) {
                result.add("MultiTouchElectrode8");
            }
		if (cfg.electrode9Enabled) {
                result.add("MultiTouchElectrode9");
            }
		if (cfg.electrode10Enabled) {
                result.add("MultiTouchElectrode10");
            }
		if (cfg.electrode11Enabled) {
                result.add("MultiTouchElectrode11");
            }
		if (cfg.proximityEnabled) {
                result.add("MultiTouchProximity");
            }
		result.add("MultiTouchRecalibrate");
        return result;
    }

    public static ChannelType getChannelType(ChannelTypeUID channelTypeUID) {
        switch(channelTypeUID.getId()) {
            case "MultiTouchElectrode":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "MultiTouchElectrode"), "NOT USED", "Switch").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:MultiTouchElectrode")).withDescription("The current touch state. An electrode is already counted as touched if a finger is nearly touching the electrode. This means that you can put a piece of paper or foil or similar on top of a electrode to build a touch panel with a professional look.").build();
            case "MultiTouchRecalibrate":
                return ChannelTypeBuilder.state(new ChannelTypeUID("tinkerforge", "MultiTouchRecalibrate"), "Recalibrate Electrodes", "String").withConfigDescriptionURI(URI.create("channel-type:tinkerforge:MultiTouchRecalibrate")).withDescription("Recalibrates the electrodes. Trigger this channel whenever you changed or moved you electrodes.")
.withCommandDescription(CommandDescriptionBuilder.create().withCommandOption(new CommandOption("TRIGGER", "Trigger")).build()).build();
            default:
                static_logger.debug("Unknown channel type ID {}", channelTypeUID.getId());
                break;
        }

        return null;
    }

    public static ThingType getThingType(ThingTypeUID thingTypeUID) {
        return ThingTypeBuilder.instance(thingTypeUID, "Tinkerforge Multi Touch Bricklet").isListed(false).withSupportedBridgeTypeUIDs(Arrays.asList(TinkerforgeBindingConstants.THING_TYPE_BRICK_DAEMON.getId())).withConfigDescriptionURI(URI.create("thing-type:tinkerforge:" + thingTypeUID.getId())).withDescription("Capacitive touch sensor for 12 electrodes").withChannelDefinitions(Arrays.asList(new ChannelDefinitionBuilder("MultiTouchElectrode0", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 0").build(), new ChannelDefinitionBuilder("MultiTouchElectrode1", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 1").build(), new ChannelDefinitionBuilder("MultiTouchElectrode2", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 2").build(), new ChannelDefinitionBuilder("MultiTouchElectrode3", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 3").build(), new ChannelDefinitionBuilder("MultiTouchElectrode4", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 4").build(), new ChannelDefinitionBuilder("MultiTouchElectrode5", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 5").build(), new ChannelDefinitionBuilder("MultiTouchElectrode6", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 6").build(), new ChannelDefinitionBuilder("MultiTouchElectrode7", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 7").build(), new ChannelDefinitionBuilder("MultiTouchElectrode8", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 8").build(), new ChannelDefinitionBuilder("MultiTouchElectrode9", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 9").build(), new ChannelDefinitionBuilder("MultiTouchElectrode10", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 10").build(), new ChannelDefinitionBuilder("MultiTouchElectrode11", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Electrode 11").build(), new ChannelDefinitionBuilder("MultiTouchProximity", new ChannelTypeUID("tinkerforge", "MultiTouchElectrode")).withLabel("Proximity").withDescription("The current touch state. If a hand or similar is in proximity to the electrodes, this channel is toggled. The proximity is activated with a distance of 1-2cm. This means that you can put a piece of paper or foil or similar on top of a electrode to build a touch panel with a professional look.").build(), new ChannelDefinitionBuilder("MultiTouchRecalibrate", new ChannelTypeUID("tinkerforge", "MultiTouchRecalibrate")).build())).build();
    }

    public static ConfigDescription getConfigDescription(URI uri) {
        switch(uri.toASCIIString()) {
            case "thing-type:tinkerforge:multitouch":
                return new ConfigDescription(uri, Arrays.asList(ConfigDescriptionParameterBuilder.create("sensitivity", Type.INTEGER).withDefault("181").withDescription("The sensitivity of the electrodes. An electrode with a high sensitivity will register a touch earlier then an electrode with a low sensitivity.<br/><br/>If you build a big electrode you might need to decrease the sensitivity, since the area that can be charged will get bigger. If you want to be able to activate an electrode from further away you need to increase the sensitivity.").withLabel("Sensitivity").withMinimum(BigDecimal.valueOf(5)).withMaximum(BigDecimal.valueOf(201)).build(), ConfigDescriptionParameterBuilder.create("proximityEnabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the proximity feature, false disables it. It is recommended that you disable the proximity feature if not needed. This will reduce the amount of traffic that is produced.").withLabel("Proximity Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode0Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 0 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode1Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 1 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode2Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 2 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode3Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 3 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode4Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 4 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode5Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 5 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode6Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 6 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode7Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 7 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode8Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 8 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode9Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 9 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode10Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 10 Enabled").build(), ConfigDescriptionParameterBuilder.create("electrode11Enabled", Type.BOOLEAN).withDefault("true").withDescription("True enables the electrode, false disables the electrode. A disabled electrode will always return false as its state. If you don't need all electrodes you can disable the electrodes that are not needed. Disabling electrodes will also reduce power consumption.").withLabel("Electrode 11 Enabled").build()), Arrays.asList(new ConfigDescriptionParameterGroup("update_intervals", "null", true, "Update Intervals", "Update Intervals")));
            case "channel-type:tinkerforge:MultiTouchElectrode":
                return new ConfigDescription(uri, Arrays.asList(ConfigDescriptionParameterBuilder.create("updateInterval", Type.INTEGER).withDefault("1000").withDescription("Specifies the update interval in milliseconds. A value of 0 disables automatic updates.").withGroupName("update_intervals").withLabel("Update Interval").withUnit("ms").build()));
            case "channel-type:tinkerforge:MultiTouchRecalibrate":
                return new ConfigDescription(uri, Arrays.asList());
            default:
                static_logger.debug("Unknown config description URI {}", uri.toASCIIString());
                break;
        }
        return null;
    }

        @Override
    public void refreshValue(String value, org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, BiConsumer<String, org.eclipse.smarthome.core.types.State> updateStateFn, BiConsumer<String, String> triggerChannelFn) throws TinkerforgeException {
        BrickletMultiTouchConfig cfg = (BrickletMultiTouchConfig) config.as(BrickletMultiTouchConfig.class);
        switch(value) {
            case "MultiTouchElectrode0": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode0Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode1": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode1Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode2": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode2Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode3": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode3Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode4": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode4Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode5": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode5Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode6": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode6Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode7": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode7Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode8": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode8Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode9": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode9Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode10": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode10Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchElectrode11": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchElectrode11Getter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchProximity": {
                   BrickletMultiTouchElectrodeConfig channelCfg = channelConfig.as(BrickletMultiTouchElectrodeConfig.class);
                   updateStateFn.accept(value, transformMultiTouchProximityGetter0(this.getTouchState(), cfg));
                   return;
               }
            case "MultiTouchRecalibrate":
               return;
            default:
                logger.warn("Refresh for unknown channel {}", value);
                break;
        }
    }
    

        @Override
    public List<SetterRefresh> handleCommand(org.eclipse.smarthome.config.core.Configuration config, org.eclipse.smarthome.config.core.Configuration channelConfig, String channel, Command command) throws TinkerforgeException {
        List<SetterRefresh> result = Collections.emptyList();
        BrickletMultiTouchConfig cfg = (BrickletMultiTouchConfig) config.as(BrickletMultiTouchConfig.class);
        switch(channel) {
            case "MultiTouchRecalibrate":
                if (command instanceof StringType) {
                    BrickletMultiTouchRecalibrateConfig channelCfg = channelConfig.as(BrickletMultiTouchRecalibrateConfig.class);
                    StringType cmd = (StringType) command;
                    this.recalibrate();
                    
                } else {
                    logger.warn("Command type {} not supported for channel {}. Please use a StringType.", command.getClass().getName(), channel);
                }
                break;
            default:
                logger.warn("Command for unknown channel {}", channel);
        }
        return result;
    }

        private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode0Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 0)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode1Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 1)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode2Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 2)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode3Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 3)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode4Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 4)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode5Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 5)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode6Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 6)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode7Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 7)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode8Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 8)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode9Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 9)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode10Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 10)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode11Callback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 11)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchProximityCallback0(int state, BrickletMultiTouchConfig cfg) {
        return (state & (1 << 12)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode0Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 0)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode1Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 1)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode2Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 2)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode3Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 3)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode4Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 4)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode5Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 5)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode6Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 6)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode7Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 7)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode8Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 8)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode9Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 9)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode10Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 10)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchElectrode11Getter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 11)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
	    private org.eclipse.smarthome.core.types.State transformMultiTouchProximityGetter0(int value, BrickletMultiTouchConfig cfg) {
        return (value & (1 << 12)) > 0 ? OnOffType.ON : OnOffType.OFF;
    }
}
